@@ -6,6 +6,7 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
+	"runtime"
 
 	"github.com/constabulary/gb"
 	"github.com/constabulary/gb/cmd"
@@ -19,8 +20,8 @@ var (
 )
 
 const (
-	// enable to keep working directory
-	noDestroyContext = false
+	// disable to keep working directory
+	destroyContext = true
 )
 
 func init() {
@@ -36,21 +37,31 @@ func registerCommand(command *cmd.Command) {
 	commands[command.Name] = command
 }
 
+// atExit functions are called in sequence at the exit of the program.
+var atExit []func() error
+
+// exit runs all atExit functions, then calls os.Exit(code).
+func exit(code int) {
+	for _, fn := range atExit {
+		fn()
+	}
+	os.Exit(code)
+}
+
 func fatalf(format string, args ...interface{}) {
 	fmt.Fprintf(os.Stderr, "FATAL: "+format+"\n", args...)
-	os.Exit(1)
+	exit(1)
 }
 
 func main() {
 	args := os.Args
 	if len(args) < 2 || args[1] == "-h" {
-		fs.Usage()
-		os.Exit(1)
+		fs.Usage() // usage calles exit(2)
 	}
 	name := args[1]
 	if name == "help" {
 		help(args[2:])
-		return
+		exit(0)
 	}
 
 	command, ok := commands[name]
@@ -58,8 +69,7 @@ func main() {
 		plugin, err := lookupPlugin(name)
 		if err != nil {
 			fmt.Fprintf(os.Stderr, "FATAL: unknown command %q\n", name)
-			fs.Usage()
-			os.Exit(1)
+			fs.Usage() // usage calles exit(2)
 		}
 		command = &cmd.Command{
 			Run: func(ctx *gb.Context, args []string) error {
@@ -82,7 +92,7 @@ func main() {
 				return cmd.Run()
 			},
 			// plugin should not interpret arguments
-			ParseArgs: func(_ *gb.Context, _ string, args []string) []string { return args },
+			SkipParseArgs: true,
 		}
 	}
 
@@ -102,6 +112,9 @@ func main() {
 	}
 
 	args = fs.Args() // reset args to the leftovers from fs.Parse
+
+	debug.Debugf("args: %v", args)
+
 	if command == commands["plugin"] {
 		args = append([]string{name}, args...)
 	}
@@ -116,27 +129,48 @@ func main() {
 		gb.Gcflags(gcflags...),
 		gb.Ldflags(ldflags...),
 		gb.Tags(buildtags...),
+		func(c *gb.Context) error {
+			if !race {
+				return nil
+			}
+
+			// check this is a supported platform
+			if runtime.GOARCH != "amd64" {
+				fatalf("race detector not supported on %s/%s", runtime.GOOS, runtime.GOARCH)
+			}
+			switch runtime.GOOS {
+			case "linux", "windows", "darwin", "freebsd":
+				// supported
+			default:
+				fatalf("race detector not supported on %s/%s", runtime.GOOS, runtime.GOARCH)
+			}
+
+			// check the race runtime is built
+			_, err := os.Stat(filepath.Join(runtime.GOROOT(), "pkg", fmt.Sprintf("%s_%s_race", runtime.GOOS, runtime.GOARCH), "runtime.a"))
+			if os.IsNotExist(err) || err != nil {
+				fatalf("go installation at %s is missing race support. See https://getgb.io/faq/#missing-race-support", runtime.GOROOT())
+			}
+
+			return gb.WithRace(c)
+		},
 	)
+
 	if err != nil {
 		fatalf("unable to construct context: %v", err)
 	}
 
-	if !noDestroyContext {
-		defer ctx.Destroy()
-	}
-
-	if command.ParseArgs != nil {
-		args = command.ParseArgs(ctx, ctx.Projectdir(), args)
-	} else {
-		args = cmd.ImportPaths(ctx, cwd, args)
+	if !command.SkipParseArgs {
+		args = importPaths(ctx, cwd, args)
 	}
 
 	debug.Debugf("args: %v", args)
+
+	if destroyContext {
+		atExit = append(atExit, ctx.Destroy)
+	}
+
 	if err := command.Run(ctx, args); err != nil {
-		if !noDestroyContext {
-			defer ctx.Destroy()
-		}
 		fatalf("command %q failed: %v", name, err)
 	}
-	return
+	exit(0)
 }
